/* hw/s3c2410x.c
 *
 * Samsung S3C2410X emulation
 *
 * Copyright 2006, 2007 Daniel Silverstone and Vincent Sanders
 *
 * This file is under the terms of the GNU General Public
 * License Version 2
 */

#include "vl.h"
#include "s3c2410x.h"

CPUState *s3c_env;
qemu_irq *s3c_irqs;
qemu_irq *s3c_eirqs;

/* Initialise a Samsung S3C2410X and all its internal peripherals */
CPUState *
s3c2410x_init(int ram_size)
{
  CPUState *env;
  qemu_irq *interrupts, *irq;
  /* Prepare the ARM 920T */
  s3c_env = env = cpu_init();
  cpu_arm_set_model(env, "arm920t");
  /* S3C2410X memory is always at the same physical location */
  S3C2410X_DBF("RAM\n");
  cpu_register_physical_memory(CPU_S3C2410X_RAM, ram_size, IO_MEM_RAM);
  /* We get a memc */
  S3C2410X_DBF("memc\n");
  s3c2410x_memc_init();
  /* And some interrupts (coo) */
  S3C2410X_DBF("interrupts\n");
  s3c_irqs = interrupts = irq = s3c2410x_irq_init(env);
  /* The clock control allows for idling the CPU */
  s3c2410x_clkcon_init(env);
  /* And some GPIO */
  S3C2410X_DBF("gpio\n");
  s3c_eirqs = s3c2410x_gpio_init(s3c_irqs); 
  /* RTC for time */
  s3c2410x_rtc_init();
  /* And some IIC */
  s3c2410x_iic_init(interrupts[27]);
  /* And some timers */
  s3c2410x_timers_init(interrupts[14]);
  /* Serial ports */
  S3C2410X_DBF("serial\n");
  s3c2410x_serial_init(0, irq[33], irq[32], irq[33+64], irq[32+64]); /* Subsrc interrupts 1 and 0 edge and level */
  s3c2410x_serial_init(1, irq[36], irq[35], irq[36+64], irq[35+64]); /* Subsrc interrupts 4 and 3 edge and level */
  s3c2410x_serial_init(2, irq[39], irq[38], irq[39+64], irq[38+64]); /* Subsrc interrupts 7 and 6 edge and level */

  /* S3C2410X SRAM */
  cpu_register_physical_memory(CPU_S3C2410X_SRAM, 4096, ram_size | IO_MEM_RAM);
  /* A two port OHCI controller on IRQ 26 */
  usb_ohci_init_pxa(CPU_S3C2410X_OHCI, 2, -1, interrupts[26]);
  return env;
}

