/* s3c2410x/regs.h
 *
 * Samsung s3c2410x cpu register definitions
 *
 * Copyright 2006, 2007 Daniel Silverstone and Vincent Sanders
 *
 * This file is under the terms of the GNU General Public
 * License Version 2.
 */

#ifndef S3C2410X_H
#define S3C2410X_H 1

#include "hw/irq.h"

/* S3C2410X SoC IDs */

#define CPU_S3C2410X_IDENT_S3C2410X 0x32410000
#define CPU_S3C2410X_IDENT_S3C2410A 0x32410002

/* all defines are prefixed with CPU_S3C2410X_ */

#define CPU_S3C2410X_REG(x) (0x48000000 + (x))

#define CPU_S3C2410X_CS0 (0x00000000)
#define CPU_S3C2410X_CS1 (0x08000000)
#define CPU_S3C2410X_CS2 (0x10000000)
#define CPU_S3C2410X_CS3 (0x18000000)
#define CPU_S3C2410X_CS4 (0x20000000)
#define CPU_S3C2410X_CS5 (0x28000000)
#define CPU_S3C2410X_RAM (0x30000000)
#define CPU_S3C2410X_SRAM (0x40000000)
#define CPU_S3C2410X_OHCI (0x49000000)

/* Clock control */
#define CPU_S3C2410X_CLKCON 0x4C000000
/* Lock time RW */
#define CPU_S3C2410X_REG_LOCKTIME 0
/* MPLL Control RW */
#define CPU_S3C2410X_REG_MPLLCON 1
/* UPLL Control RW */
#define CPU_S3C2410X_REG_UPLLCON 2
/* Clock Generator Control RW */
#define CPU_S3C2410X_REG_CLKCON 3
/* Slow Clock Control RW */
#define CPU_S3C2410X_REG_CLKSLOW 4
/* Clock divider control RW */
#define CPU_S3C2410X_REG_CLKDIVN 5
/* CLKCON IDLE */
#define CPU_S3C2410X_REG_CLKCON_IDLE (1<<2)

/* bus width, and wait state control */
#define CPU_S3C2410X_BWSCON         CPU_S3C2410X_REG(0x0000)

/* bank zero config - note, pinstrapped from OM pins! */
#define CPU_S3C2410X_BWSCON_DW0_16  (1<<1)
#define CPU_S3C2410X_BWSCON_DW0_32  (2<<1)

/* bank one configs */
#define CPU_S3C2410X_BWSCON_DW1_8   (0<<4)
#define CPU_S3C2410X_BWSCON_DW1_16  (1<<4)
#define CPU_S3C2410X_BWSCON_DW1_32  (2<<4)
#define CPU_S3C2410X_BWSCON_WS1     (1<<6)
#define CPU_S3C2410X_BWSCON_ST1     (1<<7)

/* bank 2 configurations */
#define CPU_S3C2410X_BWSCON_DW2_8   (0<<8)
#define CPU_S3C2410X_BWSCON_DW2_16  (1<<8)
#define CPU_S3C2410X_BWSCON_DW2_32  (2<<8)
#define CPU_S3C2410X_BWSCON_WS2     (1<<10)
#define CPU_S3C2410X_BWSCON_ST2     (1<<11)

/* bank 3 configurations */
#define CPU_S3C2410X_BWSCON_DW3_8   (0<<12)
#define CPU_S3C2410X_BWSCON_DW3_16  (1<<12)
#define CPU_S3C2410X_BWSCON_DW3_32  (2<<12)
#define CPU_S3C2410X_BWSCON_WS3     (1<<14)
#define CPU_S3C2410X_BWSCON_ST3     (1<<15)

/* bank 4 configurations */
#define CPU_S3C2410X_BWSCON_DW4_8   (0<<16)
#define CPU_S3C2410X_BWSCON_DW4_16  (1<<16)
#define CPU_S3C2410X_BWSCON_DW4_32  (2<<16)
#define CPU_S3C2410X_BWSCON_WS4     (1<<18)
#define CPU_S3C2410X_BWSCON_ST4     (1<<19)

/* bank 5 configurations */
#define CPU_S3C2410X_BWSCON_DW5_8   (0<<20)
#define CPU_S3C2410X_BWSCON_DW5_16  (1<<20)
#define CPU_S3C2410X_BWSCON_DW5_32  (2<<20)
#define CPU_S3C2410X_BWSCON_WS5     (1<<22)
#define CPU_S3C2410X_BWSCON_ST5     (1<<23)

/* bank 6 configurations */
#define CPU_S3C2410X_BWSCON_DW6_8   (0<<24)
#define CPU_S3C2410X_BWSCON_DW6_16  (1<<24)
#define CPU_S3C2410X_BWSCON_DW6_32  (2<<24)
#define CPU_S3C2410X_BWSCON_WS6     (1<<26)
#define CPU_S3C2410X_BWSCON_ST6     (1<<27)

/* bank 7 configurations */
#define CPU_S3C2410X_BWSCON_DW7_8   (0<<28)
#define CPU_S3C2410X_BWSCON_DW7_16  (1<<28)
#define CPU_S3C2410X_BWSCON_DW7_32  (2<<28)
#define CPU_S3C2410X_BWSCON_WS7     (1<<30)
#define CPU_S3C2410X_BWSCON_ST7     (1<<31)


/* memory set (rom, ram) */
#define CPU_S3C2410X_BANKCON0       CPU_S3C2410X_REG(0x0004)
#define CPU_S3C2410X_BANKCON1       CPU_S3C2410X_REG(0x0008)
#define CPU_S3C2410X_BANKCON2       CPU_S3C2410X_REG(0x000C)
#define CPU_S3C2410X_BANKCON3       CPU_S3C2410X_REG(0x0010)
#define CPU_S3C2410X_BANKCON4       CPU_S3C2410X_REG(0x0014)
#define CPU_S3C2410X_BANKCON5       CPU_S3C2410X_REG(0x0018)
#define CPU_S3C2410X_BANKCON6       CPU_S3C2410X_REG(0x001C)
#define CPU_S3C2410X_BANKCON7       CPU_S3C2410X_REG(0x0020)

/* bank configuration registers */

#define CPU_S3C2410X_BANKCON_PMCnorm  (0x00)
#define CPU_S3C2410X_BANKCON_PMC4     (0x01)
#define CPU_S3C2410X_BANKCON_PMC8     (0x02)
#define CPU_S3C2410X_BANKCON_PMC16    (0x03)

/* bank configurations for banks 0..7, note banks
 * 6 and 7 have differnt configurations depending on
 * the memory type bits! */

#define CPU_S3C2410X_BANKCON_Tacp2    (0x0 << 2)
#define CPU_S3C2410X_BANKCON_Tacp3    (0x1 << 2)
#define CPU_S3C2410X_BANKCON_Tacp4    (0x2 << 2)
#define CPU_S3C2410X_BANKCON_Tacp6    (0x3 << 2)

#define CPU_S3C2410X_BANKCON_Tcah0    (0x0 << 4)
#define CPU_S3C2410X_BANKCON_Tcah1    (0x1 << 4)
#define CPU_S3C2410X_BANKCON_Tcah2    (0x2 << 4)
#define CPU_S3C2410X_BANKCON_Tcah4    (0x3 << 4)

#define CPU_S3C2410X_BANKCON_Tcoh0    (0x0 << 6)
#define CPU_S3C2410X_BANKCON_Tcoh1    (0x1 << 6)
#define CPU_S3C2410X_BANKCON_Tcoh2    (0x2 << 6)
#define CPU_S3C2410X_BANKCON_Tcoh4    (0x3 << 6)

#define CPU_S3C2410X_BANKCON_Tacc1     (0x0 << 8)
#define CPU_S3C2410X_BANKCON_Tacc2     (0x1 << 8)
#define CPU_S3C2410X_BANKCON_Tacc3     (0x2 << 8)
#define CPU_S3C2410X_BANKCON_Tacc4     (0x3 << 8)
#define CPU_S3C2410X_BANKCON_Tacc6     (0x4 << 8)
#define CPU_S3C2410X_BANKCON_Tacc8     (0x5 << 8)
#define CPU_S3C2410X_BANKCON_Tacc10    (0x6 << 8)
#define CPU_S3C2410X_BANKCON_Tacc14    (0x7 << 8)

#define CPU_S3C2410X_BANKCON_Tcos0     (0x0 << 11)
#define CPU_S3C2410X_BANKCON_Tcos1     (0x1 << 11)
#define CPU_S3C2410X_BANKCON_Tcos2     (0x2 << 11)
#define CPU_S3C2410X_BANKCON_Tcos4     (0x3 << 11)

#define CPU_S3C2410X_BANKCON_Tacs0     (0x0 << 13)
#define CPU_S3C2410X_BANKCON_Tacs1     (0x1 << 13)
#define CPU_S3C2410X_BANKCON_Tacs2     (0x2 << 13)
#define CPU_S3C2410X_BANKCON_Tacs4     (0x3 << 13)

#define CPU_S3C2410X_BANKCON_SRAM      (0x0 << 15)
#define CPU_S3C2410X_BANKCON_SDRAM     (0x3 << 15)

/* next bits only for SDRAM in 6,7 */
#define CPU_S3C2410X_BANKCON_Trdc2     (0x00 << 2)
#define CPU_S3C2410X_BANKCON_Trdc3     (0x01 << 2)
#define CPU_S3C2410X_BANKCON_Trdc4     (0x02 << 2)

/* control column address select */
#define CPU_S3C2410X_BANKCON_SCANb8    (0x00 << 0)
#define CPU_S3C2410X_BANKCON_SCANb9    (0x01 << 0)
#define CPU_S3C2410X_BANKCON_SCANb10   (0x02 << 0)

#define CPU_S3C2410X_REFRESH        CPU_S3C2410X_REG(0x0024)
#define CPU_S3C2410X_BANKSIZE       CPU_S3C2410X_REG(0x0028)
#define CPU_S3C2410X_MRSRB6         CPU_S3C2410X_REG(0x002C)
#define CPU_S3C2410X_MRSRB7         CPU_S3C2410X_REG(0x0030)

/* mode select register(s) */

#define  CPU_S3C2410X_MRSRB_CL1     (0x00 << 4)
#define  CPU_S3C2410X_MRSRB_CL2     (0x02 << 4)
#define  CPU_S3C2410X_MRSRB_CL3     (0x03 << 4)

/* bank size register */
#define CPU_S3C2410X_BANKSIZE_128M  (0x2 << 0)
#define CPU_S3C2410X_BANKSIZE_64M   (0x1 << 0)
#define CPU_S3C2410X_BANKSIZE_32M   (0x0 << 0)
#define CPU_S3C2410X_BANKSIZE_16M   (0x7 << 0)
#define CPU_S3C2410X_BANKSIZE_8M    (0x6 << 0)
#define CPU_S3C2410X_BANKSIZE_4M    (0x5 << 0)
#define CPU_S3C2410X_BANKSIZE_2M    (0x4 << 0)

/* interrupt controller */

#define CPU_S3C2410X_SRCPND         CPU_S3C2410X_REG(0x02000000)
#define CPU_S3C2410X_INTMOD         CPU_S3C2410X_REG(0x02000004)
#define CPU_S3C2410X_INTMSK         CPU_S3C2410X_REG(0x02000008)
#define CPU_S3C2410X_PRIORITY       CPU_S3C2410X_REG(0x0200000C)
#define CPU_S3C2410X_INTPND         CPU_S3C2410X_REG(0x02000010)
#define CPU_S3C2410X_INTOFFSET      CPU_S3C2410X_REG(0x02000014)
#define CPU_S3C2410X_SUBSRCPND      CPU_S3C2410X_REG(0x02000018)
#define CPU_S3C2410X_INTSUBMSK      CPU_S3C2410X_REG(0x0200001C)

/* serial ports */

#define CPU_S3C2410X_SERIAL_BASE(port) (0x50000000 + (port * 0x4000))
/* Line control         RW WORD */
#define CPU_S3C2410X_SERIAL_ULCON 0x00
/* General control      RW WORD */
#define CPU_S3C2410X_SERIAL_UCON  0x04
/* Fifo control         RW WORD */
#define CPU_S3C2410X_SERIAL_UFCON 0x08
/* Modem control        RW WORD */
#define CPU_S3C2410X_SERIAL_UMCON 0x0C
/* TX/RX Status         RO WORD */
#define CPU_S3C2410X_SERIAL_UTRSTAT 0x10
/* Receive Error Status RO WORD */
#define CPU_S3C2410X_SERIAL_UERSTAT 0x14
/* FiFo Status          RO WORD */
#define CPU_S3C2410X_SERIAL_UFSTAT 0x18
/* Modem Status         RO WORD */
#define CPU_S3C2410X_SERIAL_UMSTAT 0x1C
/* TX buffer            WR BYTE */
#define CPU_S3C2410X_SERIAL_UTXH 0x20
/* RX buffer            RO BYTE */
#define CPU_S3C2410X_SERIAL_URXH 0x24
/* BAUD Divisor         RW WORD */
#define CPU_S3C2410X_SERIAL_UBRDIV 0x28

/* LCD */
#define CPU_S3C2410X_LCD_BASE	0x4D000000

/* IIC */
#define CPU_S3C2410X_IIC_BASE	0x54000000

/* Watchdog */
#define CPU_S3C2410X_WDOG_BASE	0x53000000

/* NAND */
#define CPU_S3C2410X_NAND_BASE	0x4E000000

/* GPIO */
#define CPU_S3C2410X_GPIO_BASE	0x56000000

/* Interrupt controller */
#define CPU_S3C2410X_IRQ_BASE 0x4A000000
/* IRQ request status              RW WORD */
#define CPU_S3C2410X_IRQ_SRCPND 0
/* Interrupt mode control          WR WORD */
#define CPU_S3C2410X_IRQ_INTMOD 1
/* Interrupt mask control          RW WORD */
#define CPU_S3C2410X_IRQ_INTMSK 2
/* IRQ priority control            WR WORD */
#define CPU_S3C2410X_IRQ_PRIORITY 3
/* Interrupt request status        RW WORD */
#define CPU_S3C2410X_IRQ_INTPND 4
/* Interrupt request source offset RO WORD */
#define CPU_S3C2410X_IRQ_OFFSET 5
/* Sub-source pending              RW WORD */
#define CPU_S3C2410X_IRQ_SUBSRCPND 6
/* Interrupt sub-mask              RW WORD */
#define CPU_S3C2410X_IRQ_INTSUBMSK 7

/* Timers */

#define CPU_S3C2410X_TIMERS_BASE 0x51000000
/* Timer configuration 0 */
#define CPU_S3C2410X_TIMERS_TCFG0 0
/* Timer configuration 1 */
#define CPU_S3C2410X_TIMERS_TCFG1 1
/* Timer control */
#define CPU_S3C2410X_TIMERS_TCON 2
/* Timer count buffer 0 */
#define CPU_S3C2410X_TIMERS_TCNTB0 3
/* Timer compare buffer 0 */
#define CPU_S3C2410X_TIMERS_TCMPB0 4
/* Timer count observation 0 */
#define CPU_S3C2410X_TIMERS_TCNTO0 5
/* Timer count buffer 1 */
#define CPU_S3C2410X_TIMERS_TCNTB1 6
/* Timer compare buffer 1 */
#define CPU_S3C2410X_TIMERS_TCMPB1 7
/* Timer count observation 1 */
#define CPU_S3C2410X_TIMERS_TCNTO1 8
/* Timer count buffer 2 */
#define CPU_S3C2410X_TIMERS_TCNTB2 9
/* Timer compare buffer 2 */
#define CPU_S3C2410X_TIMERS_TCMPB2 10
/* Timer count observation 2 */
#define CPU_S3C2410X_TIMERS_TCNTO2 11
/* Timer count buffer 3 */
#define CPU_S3C2410X_TIMERS_TCNTB3 12
/* Timer compare buffer 3 */
#define CPU_S3C2410X_TIMERS_TCMPB3 13
/* Timer count observation 3 */
#define CPU_S3C2410X_TIMERS_TCNTO3 14
/* Timer count buffer 4 */
#define CPU_S3C2410X_TIMERS_TCNTB4 15
/* Timer count observation 4 */
#define CPU_S3C2410X_TIMERS_TCNTO4 16

/* Real time clock */
#define CPU_S3C2410X_RTC_BASE 0x57000040
/* RTC Control RW Byte */
#define CPU_S3C3410X_REG_RTCCON 0
/* Tick time count RW Byte */
#define CPU_S3C2410X_REG_TICNT 1
/* RTC Alarm Control RW Byte */
#define CPU_S3C2410X_REG_RTCALM 4
/* Alarm second */
#define CPU_S3C2410X_REG_ALMSEC 5
/* Alarm minute */
#define CPU_S3C2410X_REG_ALMMIN 6
/* Alarm hour */
#define CPU_S3C2410X_REG_ALMHOUR 7
/* Alarm day */
#define CPU_S3C2410X_REG_ALMDATE 8
/* Alarm month */
#define CPU_S3C2410X_REG_ALMMON 9
/* Alarm year */
#define CPU_S3C2410X_REG_ALMYEAR 10
/* RTC Round Reset */
#define CPU_S3C2410X_REG_RTCRST 11
/* BCD Second */
#define CPU_S3C2410X_REG_BCDSEC 12
/* BCD Minute */
#define CPU_S3C2410X_REG_BCDMIN 13
/* BCD Hour */
#define CPU_S3C2410X_REG_BCDHOUR 14
/* BCD Day */
#define CPU_S3C2410X_REG_BCDDATE 15
/* BCD Day of week */
#define CPU_S3C2410X_REG_BCDDAY 16
/* BCD Month */
#define CPU_S3C2410X_REG_BCDMON 17
/* BCD Year */
#define CPU_S3C2410X_REG_BCDYEAR 18

/* IRQ state */
extern qemu_irq *s3c_irqs;

/* Functions */
CPUState *s3c2410x_init(int ram_size);

void s3c2410x_memc_init(void);
qemu_irq *s3c2410x_irq_init(CPUState *env);
qemu_irq *s3c2410x_gpio_init(qemu_irq *real_irqs);
void s3c2410x_serial_init(int port, qemu_irq tx_irq, qemu_irq rx_irq, qemu_irq tx_level, qemu_irq rx_level);
void s3c2410x_clkcon_init(CPUState *ce);
void s3c2410x_timers_init(qemu_irq timer4_irq);
void s3c2410x_iic_init(qemu_irq iic_irq);
void s3c2410x_rtc_init(void);
void s3c2410x_lcd_init(void);

int s3c2410_nand_load(const char *fname,
			     unsigned int id1, unsigned id2,
			     unsigned int oob);

#ifdef DEBUG_S3C2410X_INLINE
#define S3C2410X_DBF_STREAM stdout
#else
#define S3C2410X_DBF_STREAM stderr
#endif
#if defined(DEBUG_S3C2410X) || 0
#define S3C2410X_DBF(X...) fprintf(S3C2410X_DBF_STREAM,"S3C2410X:" X)
#else
#define S3C2410X_DBF(X...) if(0) (fprintf)(S3C2410X_DBF_STREAM, X)
#endif

#endif /* S3C2410X_H */
