/* hw/s3c2410x_clkcon.c
 *
 * Samsung S3C2410X emulation
 *
 * Copyright 2006, 2007 Daniel Silverstone and Vincent Sanders
 *
 * This file is under the terms of the GNU General Public
 * License Version 2
 */

#include "vl.h"
#include "s3c2410x.h"

CPUState *env;

static uint32_t s3c2410x_clkcon_reg[6];
#define CR(X) s3c2410x_clkcon_reg[X]

static void
s3c2410x_clkcon_write_f(void *opaque_ignored, target_phys_addr_t addr_, uint32_t value)
{
  int addr = (addr_ & 0x1F) >> 2;
  int idle_rising_edge = 0;
  if (addr < 0 || addr > 5) addr = 5;

  S3C2410X_DBF("Write CLKCON[%02x] = %08x\n", addr_&0xFF, value);

  if( addr == CPU_S3C2410X_REG_CLKCON ) {
    if( !(s3c2410x_clkcon_reg[addr] & CPU_S3C2410X_REG_CLKCON_IDLE) &&
        (value & CPU_S3C2410X_REG_CLKCON_IDLE) ) idle_rising_edge = 1;
  }
  s3c2410x_clkcon_reg[addr] = value;
  if( idle_rising_edge ) {
    S3C2410X_DBF("CLKCON: Idling CPU\n");
    env->exception_index = EXCP_HLT;
    env->halted = 1;
    cpu_loop_exit();
  }
}

static uint32_t
s3c2410x_clkcon_read_f(void *opaque_ignored, target_phys_addr_t addr_)
{
  int addr = (addr_ & 0x1F) >> 2;
  if (addr < 0 || addr > 5) addr = 5;
  S3C2410X_DBF("Read CLKCON[%d]\n", addr);
  return s3c2410x_clkcon_reg[addr];
}

static CPUReadMemoryFunc *s3c2410x_clkcon_read[] = {
  &s3c2410x_clkcon_read_f,
  &s3c2410x_clkcon_read_f,
  &s3c2410x_clkcon_read_f,
};

static CPUWriteMemoryFunc *s3c2410x_clkcon_write[] = {
  &s3c2410x_clkcon_write_f,
  &s3c2410x_clkcon_write_f,
  &s3c2410x_clkcon_write_f,
};


void
s3c2410x_clkcon_init(CPUState *ce)
{
  int tag;

  tag = cpu_register_io_memory(0, s3c2410x_clkcon_read, s3c2410x_clkcon_write, 0);
  cpu_register_physical_memory(CPU_S3C2410X_CLKCON, 6*4, tag);
  
  CR(0) = 0x00FFFFFF;
  CR(1) = 0x0005C080;
  CR(2) = 0x00028080;
  CR(3) = 0x0007FFF0;
  CR(4) = 0x00000004;
  CR(5) = 0x00000000;
  
  env = ce;
}
