/* hw/s3c2410x_rtc.c
 *
 * Samsung S3C2410X emulation
 *
 * Copyright 2006, 2007 Daniel Silverstone and Vincent Sanders
 *
 * This file is under the terms of the GNU General Public
 * License Version 2
 */

#include "vl.h"
#include "s3c2410x.h"

static uint8_t s3c2410x_rtc_reg[19];

static inline int to_bcd(int a)
{
  return ((a/10)<<4) | (a%10);
}

static void
s3c2410x_rtc_write_f(void *opaque_ignored, target_phys_addr_t addr_, uint32_t value)
{
  int addr = (addr_ - CPU_S3C2410X_RTC_BASE) >> 2;
  if (addr < 0 || addr > 18) addr = 18;
  S3C2410X_DBF("Write RTC[%d] = %08x\n", addr, value);
  s3c2410x_rtc_reg[addr] = value;
}

static uint32_t
s3c2410x_rtc_read_f(void *opaque_ignored, target_phys_addr_t addr_)
{
  int addr = (addr_ - CPU_S3C2410X_RTC_BASE) >> 2;
  struct tm *tm;
  time_t ti;
  if (addr < 0 || addr > 18) addr = 18;
  S3C2410X_DBF("Read RTC[%d]\n", addr);
  if( addr >= CPU_S3C2410X_REG_BCDSEC &&
      addr <= CPU_S3C2410X_REG_BCDYEAR ) {
    time(&ti);
    if( rtc_utc )
      tm = gmtime(&ti);
    else
      tm = localtime(&ti);
    switch(addr) {
    case CPU_S3C2410X_REG_BCDSEC:
      return to_bcd(tm->tm_sec);
    case CPU_S3C2410X_REG_BCDMIN:
      return to_bcd(tm->tm_min);
    case CPU_S3C2410X_REG_BCDHOUR:
      return to_bcd(tm->tm_hour);
    case CPU_S3C2410X_REG_BCDDATE:
      return to_bcd(tm->tm_mday);
    case CPU_S3C2410X_REG_BCDDAY:
      return to_bcd(tm->tm_wday+1);
    case CPU_S3C2410X_REG_BCDMON:
      return to_bcd(tm->tm_mon+1);
    case CPU_S3C2410X_REG_BCDYEAR:
      return to_bcd(tm->tm_year-100);
    }
  }
  return s3c2410x_rtc_reg[addr];
}

static CPUReadMemoryFunc *s3c2410x_rtc_read[] = {
  &s3c2410x_rtc_read_f,
  &s3c2410x_rtc_read_f,
  &s3c2410x_rtc_read_f,
};

static CPUWriteMemoryFunc *s3c2410x_rtc_write[] = {
  &s3c2410x_rtc_write_f,
  &s3c2410x_rtc_write_f,
  &s3c2410x_rtc_write_f,
};


void
s3c2410x_rtc_init(void)
{
  int tag;
  tag = cpu_register_io_memory(0, s3c2410x_rtc_read, s3c2410x_rtc_write, 0);
  cpu_register_physical_memory(CPU_S3C2410X_RTC_BASE, 19*4, tag);
  for(tag=0; tag < 18; ++tag) s3c2410x_rtc_reg[tag] = 0;
}
